/**
 * FlowSlicer - visualizing 3-D flow data.  Written by Matt Ward
 * 
 * Loads the file uvw.dat, which is an ascii file with about 400K 3-D vectors.
 * The format is [x y z u v w] in floating point.  The output is an animation
 * of the slices on a uniform grid with 2 of the 3 vector components creating a
 * line in the direction of the vector.
 */

String[] lines; // lines of the input data
float[][][] u, v, w; // storage for the 3-D vectors
int slice = 0;  // slice of the volume being visualized

void setup() {
  size(800, 800);
  background(0);
  stroke(255);
  frameRate(1); // put a little space between frams
  lines = loadStrings("uvw.dat");  // data set is hard coded
  u = new float[96][65][48];  // as are the dimensions!
  v = new float[96][65][48];
  w = new float[96][65][48];
  int index = 2;  // first 2 lines are metadata
  for(int i = 0; i < 48; i++)
    for(int j = 0;j < 65;j++)
      for(int k = 0;k < 96; k++)  {
        String[] pieces = split(lines[index], ' ');  // break line into tokens
        u[k][j][i] = float(pieces[3]) * 5.;  // note dx/u is scaled different from others
        v[k][j][i] = float(pieces[4]) * 100.;
        w[k][j][i] = float(pieces[5]) * 100.;
        index++;  // get the next line of data
      }
}

// draw the current slice
void draw() {
  int sx, sy, ex, ey;
  background(0);  // clear the display first
  for(int j = 0;j < 65;j++)
    for(int k = 0; k < 96; k++)  {
      sx = j * 8;  // I space the points out so vectors don't overlap
      sy = k * 8;
      ex = sx + (int)u[k][j][slice];  // endpoint is based on 2 vector components
      ey = sy + (int)w[k][j][slice];
      line(sx, sy, ex, ey);
    }
   slice++;  // increment the slice - if the last one, start over
   if(slice >= 48) {
     slice = 0;
   }
}

